part of dartrocket;

/**
 * State is an absract class that represents a state of the game.
 * 
 * For using it you need to extend the state and call it's constructor, after that
 * you have to owerwrite the run() methode.
 * 
 * There 2 **important** methode that controls the state:
 * 
 * * endState: This methodes end the state, but we can come back to this state
 * * killState: This kills that state, so you can not return back to this state
 * 
 * **Important:** After you call terminateState you can never go back to this state.
 * If you want to return to a state use endState method.
 * 
 * **Important:** You can controll where you want to go with nextState method.
 * 
 * Example:
 *     class MyState extends State{
 *       //You can genrate it with DartEditor's quikfix
 *       MyState(String name, [String nextState]): super(name, nextState);
 *       
 *       load(){
 *        //loading stuff
 *       }
 *       
 *       run(){         
 *       
 *         //running stuff
 *         if(playerLost){
 *           //menu will be the next state and we can try again
 *           nextState = "menu";
 *           endState(); 
 *         }else{
 *           //player won so we go to the next level and never return
 *           nextState = "nextLevel";
 *           terminateState();
 *         }
 *         
 *       }
 *     }
 * 
 * */

abstract class State extends Stream<String> {

  static const String _PAUSE = "PAUSE";

  StreamController<String> _controller;
  String _name;
  /**
   * The next state's name.
   * 
   * **Important:** You can controll where you want to go with this member.
   */
  String nextState;
  /**
   * Acces to the main game object.
   * */
  Game game;

  /**
   * This will make a state.
   * 
   * Parameters:
   * 
   * * _name: the state's name
   * * nextState: next state's name(optional)
   * */
  State(this._name, [String nextState = null]): nextState = nextState {
    _controller = new StreamController<String>(onListen: _onListen, onPause:
        _onPause, onResume: _onResume, onCancel: _onCancel);
  }
  /**
   * This methode can only be used by the [StateManager]'s object.
   * 
   * **DO NOT USE IT.**
   * */
  StreamSubscription<String> listen(void onData(String line), {void
      onError(Error error), void onDone(), bool cancelOnError}) {
    return _controller.stream.listen(onData, onError: onError, onDone: onDone,
        cancelOnError: cancelOnError);
  }

  void _onListen() {
    load();
    game.resourceManager.load().then((_){run();});
  }

  void _onPause() {
    game.stage.removeChildren();
    game.stage.juggler.clear();
  }

  void _onResume() {
    load();
    game.resourceManager.load().then((_){run();});
  }

  void _onCancel() {
    game.stage.removeChildren();
    game.stage.juggler.clear();
  }
  /**
   * Overwrite if you want to load resources.
   * */
  load() {}

  /**
   * You must overwrite this for using the state.
   * 
   * Called after load has completed. 
   * */
  run();

  /**
   * Gets the name of the state.
   * */
  String get name => _name;

  /**
   * You can send message to the StateManager.
   * */
  void addMessage(message) {
    _controller.add(message);
  }

  /**
   * This methodes end the state, but we can come back to this state
   * */
  void endState() {
    _controller.add(_PAUSE);
  }
  /**
   * This kills that state, so you can not return back to this state
   * */
  void killteState() {
    _controller.close();
  }


}
